package info.textgrid.lab.collatex.model.equivalences;

import java.util.Set;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import com.google.common.collect.Sets;


/**
 * A line in the equivalence table: All tokens matching at least one of the
 * tokens in the same equivalence are considered equivalent.
 * 
 * @see info.textgrid.lab.collatex.model.equivalences the package description
 *      for an detailed description of the equivalence algorithm.
 */
@XmlRootElement
public class Equivalence {

	@XmlTransient
	private boolean local;

	@XmlTransient
	private final Set<EToken> tokens = Sets.newLinkedHashSet();

	/**
	 * Whether this equivalence is local or global. For global equivalences, the
	 * token positions are not considered.
	 */
	public boolean isLocal() {
		return local;
	}

	public void setLocal(final boolean local) {
		this.local = local;
	}

	/**
	 * The set of tokens to be considered equivalent.
	 */
	@XmlElement(name = "token")
	public Set<EToken> getTokens() {
		return tokens;
	}

	@Override
	public String toString() {
		return (local ? "Local " : "Global ") + "Equivalence: " + tokens; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
}
