package info.textgrid.lab.collatex.ui.parts;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;
import org.neo4j.graphdb.Transaction;

import com.google.common.collect.Lists;
import com.google.common.collect.RowSortedTable;

import eu.interedition.collatex.Token;
import eu.interedition.collatex.Witness;
import eu.interedition.collatex.graph.VariantGraph;
import eu.interedition.collatex.graph.VariantGraphTransposition;

/**
 * @deprecated we currently use {@link GridResultView} instead.
 */
@Deprecated
public class VerticalTableView extends ViewPart {

	/**
	 * The content manager serves as both content and label provider for the
	 * viewer, and it is also responsible for managing the table's columns.
	 */
	private final class WitnessTableContentManager implements
	IStructuredContentProvider {
		private TableViewer viewer;
		private final List<TableColumn> columns = Lists.newArrayList();
		private VariantGraph variantGraph;
		private RowSortedTable<Integer, Witness, Set<Token>> alignmentTable;
		private Set<VariantGraphTransposition> transpositions;

		@Override
		public void dispose() {
			// TODO Auto-generated method stub

		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			this.viewer = (TableViewer) viewer;

			// remove old columns
			for (final Iterator<TableColumn> iterator = columns
					.iterator(); iterator.hasNext();) {
				iterator.next().dispose();
				iterator.remove();
			}

			if (newInput != null && newInput instanceof VariantGraph) {
				variantGraph = (VariantGraph) newInput;
				final Transaction transaction = variantGraph.getDatabase()
						.beginTx();
				try {
					alignmentTable = variantGraph.toTable();
					transpositions = variantGraph.transpositions();
					transaction.success();
				} finally {
					transaction.finish();
				}
				final int witnessCount = alignmentTable.columnKeySet().size();
				final int columnWidth = Math.max(50, viewer.getControl()
						.getBounds().width / witnessCount);
				for (final Witness witness : alignmentTable.columnKeySet()) {
					final TableViewerColumn viewerColumn = new TableViewerColumn(
							this.viewer, SWT.NONE);
					viewerColumn
					.setLabelProvider(new DelegatingStyledCellLabelProvider(
							new WitnessLabelProvider(null, witness)));
					viewerColumn.setEditingSupport(new EditingSupport(
							this.viewer) {

						@Override
						protected CellEditor getCellEditor(final Object element) {
							// TODO Auto-generated method stub
							return null;
						}

						@Override
						protected boolean canEdit(final Object element) {
							return true;
						}

						@Override
						protected Object getValue(final Object element) {
							// TODO Auto-generated method stub
							return null;
						}

						@Override
						protected void setValue(final Object element,
								final Object value) {
							// TODO Auto-generated method stub

						}

					});
					final TableColumn column = viewerColumn.getColumn();
					column.setText(witness.getSigil());
					column.setWidth(columnWidth);
					columns.add(column);
				}
			}
		}

		@Override
		public Object[] getElements(final Object inputElement) {
			if (inputElement == variantGraph)
				return alignmentTable.rowMap().entrySet().toArray();
			return new Object[0];
		}
	}

	public static final String ID = "info.textgrid.lab.collatex.ui.parts.VerticalTableView"; //$NON-NLS-1$
	private Table table;
	private VariantGraph variantGraph;
	private TableViewer tableViewer;
	private WitnessTableContentManager contentManager;

	public VerticalTableView() {
	}

	/**
	 * Create contents of the view part.
	 * @param parent
	 */
	@Override
	public void createPartControl(final Composite parent) {
		final Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(1, false));
		tableViewer = new TableViewer(container,
				SWT.BORDER);
		table = tableViewer.getTable();
		table.setHeaderVisible(true);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1,
				1));
		contentManager = new WitnessTableContentManager();
		tableViewer.setContentProvider(contentManager);

		createActions();
		initializeToolBar();
		initializeMenu();
	}

	/**
	 * Create the actions.
	 */
	private void createActions() {
		// Create the actions
	}

	/**
	 * Initialize the toolbar.
	 */
	private void initializeToolBar() {
		final IToolBarManager toolbarManager = getViewSite().getActionBars()
				.getToolBarManager();
	}

	/**
	 * Initialize the menu.
	 */
	private void initializeMenu() {
		final IMenuManager menuManager = getViewSite().getActionBars()
				.getMenuManager();
	}

	@Override
	public void setFocus() {
		// Set the focus
	}

	public void setInput(final VariantGraph variantGraph) {
		this.variantGraph = variantGraph;
		if (tableViewer != null && !tableViewer.getTable().isDisposed())
			tableViewer.setInput(variantGraph);
	}


}
